//      PekWM dynamic menu generator
//      
//      openbox-menu http://fabrice.thiroux.free.fr/openbox-menu_en.html 
//      by: mimas <mimasgpc@free.fr>
//      
//      modified to work with pekwm
//      author: archpup@gmail.com
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU Lesser General Public License as published
//      by the Free Software Foundation; version 3.0 only.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.


#include <menu-cache.h>
#include "pekwm-menu.h"

extern gboolean no_icon;
extern gboolean comment_name;
extern gboolean show_flag;


static void
menu_directory (MenuCacheApp *dir)
{
	gchar *dir_name = sanitize (menu_cache_item_get_name (MENU_CACHE_ITEM(dir)));
	g_print ("Submenu=\"%s\" {\n", dir_name);
	g_free (dir_name);
}


static void
menu_application (MenuCacheApp *app)
{
	gchar *exec_name = NULL;

	if (comment_name)
		exec_name = sanitize(menu_cache_item_get_comment (MENU_CACHE_ITEM(app)));

	if (exec_name == NULL)
		exec_name = sanitize(menu_cache_item_get_name (MENU_CACHE_ITEM(app)));

	gchar *exec_cmd = clean_exec (menu_cache_app_get_exec (MENU_CACHE_APP(app)));

    g_print ("Entry = \"%s\"", exec_name);
    g_print (" { Actions = \"Exec %s &\" }\n", exec_cmd);
	g_free (exec_name);
	g_free (exec_cmd);
}


/* Main loop for pekwm menu generator */
void
generate_pekwm_menu (MenuCacheDir *dir)
{
	GSList *l = NULL;

	for (l = menu_cache_dir_get_children (dir); l; l = l->next)
		switch ((guint) menu_cache_item_get_type (MENU_CACHE_ITEM(l->data)))
		{
			case MENU_CACHE_TYPE_DIR:
				menu_directory (l->data);
				generate_pekwm_menu (MENU_CACHE_DIR(l->data));
				g_print ("}\n");
				break;

			case MENU_CACHE_TYPE_SEP:
				g_print ("Separator {}\n");
				break;

			case MENU_CACHE_TYPE_APP:
				if (app_is_visible (MENU_CACHE_APP(l->data), show_flag))
					menu_application (l->data);
		}
}
